Attribute VB_Name = "basTextBoxFeatures"
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long

Private Const EM_GETLINECOUNT = &HBA
Private Const EM_GETLINE = &HC4
Private Const EM_FMTLINES = &HC8
Private Const EM_LINELENGTH = &HC1

Private Const EC_LEFTMARGIN = &H1
Private Const EC_RIGHTMARGIN = &H2
Private Const EC_USEFONTINFO = &HFFFF
Private Const EM_SETMARGINS = &HD3
Private Const EM_GETMARGINS = &HD4

Private Const EM_CANUNDO = &HC6
Private Const EM_EMPTYUNDOBUFFER = &HCD

Private Const EM_GETFIRSTVISIBLELINE = &HCE
Private Const EM_GETHANDLE = &HBD


Private Const EM_GETMODIFY = &HB8
Private Const EM_GETPASSWORDCHAR = &HD2
Private Const EM_GETRECT = &HB2
Private Const EM_GETSEL = &HB0
Private Const EM_GETTHUMB = &HBE
Private Const EM_GETWORDBREAKPROC = &HD1
Private Const EM_LIMITTEXT = &HC5
Private Const EM_LINEFROMCHAR = &HC9
Private Const EM_LINEINDEX = &HBB

Private Const EM_LINESCROLL = &HB6
Private Const EM_REPLACESEL = &HC2
Private Const EM_SCROLL = &HB5
Private Const EM_SCROLLCARET = &HB7
Private Const EM_SETHANDLE = &HBC
Private Const EM_SETMODIFY = &HB9
Private Const EM_SETPASSWORDCHAR = &HCC
Private Const EM_SETREADONLY = &HCF
Private Const EM_SETRECT = &HB3
Private Const EM_SETRECTNP = &HB4
Private Const EM_SETSEL = &HB1
Private Const EM_SETTABSTOPS = &HCB
Private Const EM_SETWORDBREAKPROC = &HD0
Private Const EM_UNDO = &HC7
' Returns the length og the current line
Public Property Get CurrentLineLength(txtBox As TextBox) As Long
    CurrentLineLength = SendMessage(txtBox.hwnd, EM_LINELENGTH, txtBox.SelStart, 0)
End Property
' Returns the length of the line at the given character index
Public Property Get LineLength(txtBox As TextBox, CharacterIndex As Long) As Long
    LineLength = SendMessage(txtBox.hwnd, EM_LINELENGTH, CharacterIndex, 0&)
End Property
' Returns true if there are items to undo
' otherwise returns false
Public Function CanUndo(txtBox As TextBox) As Boolean
    CanUndo = SendMessage(txtBox.hwnd, EM_CANUNDO, &O0, &O0)
End Function

' Clears the undo buffer
Public Sub EmptyUndo(txtBox As TextBox)
     SendMessage txtBox.hwnd, EM_EMPTYUNDOBUFFER, 0, 0
End Sub


' Returns the text at the current line
Public Property Get CurrentLine(txtBox As TextBox) As String
    CurrentLine = LineAt(txtBox, CurrentLinePos(txtBox))
End Property


'Every time the text in a textbox is changed a flag
'is set to indicate this.  This notifaction can be cleared by calling
'EM_SETMODIFY
Public Property Get isDirty(txtBox As TextBox) As Boolean
    isDirty = SendMessage(txtBox.hwnd, EM_GETMODIFY, 0, 0)
End Property


Public Property Let isDirty(txtBox As TextBox, Dirty As Boolean)
    SendMessage txtBox.hwnd, EM_SETMODIFY, Dirty, 0
End Property


'Returns the left margin of the textbox control
Public Property Get LeftMargin(txtBox As TextBox) As Integer
    LeftMargin = LoWord(SendMessage(txtBox.hwnd, EM_GETMARGINS, 0&, 0&))
End Property

'Returns the right margin of a text box
Public Property Get RightMargin(txtBox As TextBox) As Integer
    RightMargin = HiWord(SendMessage(txtBox.hwnd, EM_GETMARGINS, 0, 0))
End Property


Public Property Get LineCount(txtBox As TextBox) As Long
On Local Error Resume Next
    LineCount = SendMessage(txtBox.hwnd, EM_GETLINECOUNT, 0&, 0&)
End Property

' Returns a number indicating which line the cursor is currently on
Public Property Get CurrentLinePos(txtBox As TextBox) As Long
    CurrentLinePos = SendMessage(txtBox.hwnd, EM_LINEFROMCHAR, ByVal -1, 0&)
End Property

'Returns the line of text at a given line number
Public Function LineAt(txtBox As TextBox, LineIndex As Long) As String
    ' When sending the EM_GETLINE message
    ' the lParam is a pointer to a buffer that will hold
    ' the returned line of text.
    ' The first word(16 bits) of this buffer contain the
    ' number of characters to return.  So we can set the
    ' number of characters to return a Byte array is used
    Dim bReturnedLineBuffer() As Byte
    Dim LengthOfLine As Long ' length of the line
    Dim LineStart As Long

    LineStart = LineStartPos(txtBox, LineIndex)
    If LineStart = -1 Then Exit Function
    
    LengthOfLine = LineLength(txtBox, LineStart)
    If LengthOfLine < 1 Then Exit Function
    
    'Resize the byte array
    ReDim bReturnedLineBuffer(LengthOfLine)

    'Save the length in the first word of the array
    'A Word is two bytes so split the length up
    bReturnedLineBuffer(0) = LengthOfLine And 255
    bReturnedLineBuffer(1) = LengthOfLine \ 256


    SendMessage txtBox.hwnd, EM_GETLINE, LineIndex, bReturnedLineBuffer(0)

    'Make the byte array a string and return it
    LineAt = Left$(StrConv(bReturnedLineBuffer, vbUnicode), LengthOfLine)
End Function
' Returns the character index that starts this line
Public Property Get LineStartPos(txtBox As TextBox, ByVal LineIndex As Long) As Long
    LineStartPos = SendMessage(txtBox.hwnd, EM_LINEINDEX, LineIndex, 0&)
End Property

' Returns the character position that
' starts the current line
Public Property Get CurrentLineStartPos(txtBox As TextBox) As Long
    CurrentLineStartPos = SendMessage(txtBox.hwnd, EM_LINEINDEX, -1, 0&)
End Property

' Returns the line number that the character at CharacterIndex is on
Public Property Get LinePos(txtBox As TextBox, CharacterIndex As Long) As Long
    CurrentLine = SendMessage(txtBox.hwnd, EM_LINEFROMCHAR, CharacterIndex, 0&)
End Property

'Scrolls from the current position by the amounts specified
Public Sub Scroll(txtBox As TextBox, ByVal cxScroll As Long, ByVal cyScroll As Long)
    SendMessage txtBox.hwnd, EM_LINESCROLL, ByVal cxScroll, ByVal cyScroll
End Sub

' Makes the Caret visible
Public Sub ScrollToCaret(txtBox As TextBox)
    SendMessage txtBox.hwnd, EM_SCROLLCARET, 0&, 0&
End Sub
' Returns a number indicating the first visible line in a text box
Public Property Get FirstVisibleLine(txtBox As TextBox) As Long
    FirstVisibleLine = SendMessage(txtBox.hwnd, EM_GETFIRSTVISIBLELINE, 0&, 0&)
End Property


' Undo does last change.  The same as undoing using the popupmenu
Public Sub Undo(txtBox As TextBox)
    SendMessage txtBox.hwnd, EM_UNDO, &O0, &O0
End Sub

'Sets the left margin of a textbox control
Public Property Let LeftMargin(txtBox As TextBox, newLeftMargin As Integer)
    ' In this call the lower half of the Margins variable contains the
    ' left margin and the upper half contains the right margin
    SendMessage txtBox.hwnd, EM_SETMARGINS, EC_LEFTMARGIN, ByVal newLeftMargin
End Property


' Sets the right margin of a text box
Public Property Let RightMargin(txtBox As TextBox, newRightMargin As Integer)
    Dim Margins As Long
    ' In this call the lower half of the Margins variable contains the
    ' left margin and the upper half contains the right margin
    HiWord(Margins) = newRightMargin
    SendMessage txtBox.hwnd, EM_SETMARGINS, EC_RIGHTMARGIN, ByVal Margins
End Property

'Returns the lest significant 16 bits of a long value
Public Property Get LoWord(ByRef lThis As Long) As Long
   LoWord = (lThis And &HFFFF&)
End Property

'Sets the lest significant 16 bits of a long value
Public Property Let LoWord(ByRef lThis As Long, ByVal lLoWord As Long)
   lThis = lThis And Not &HFFFF& Or lLoWord
End Property

'Returns the most significant 16 bits of a long value
Public Property Get HiWord(ByRef lThis As Long) As Long
   If (lThis And &H80000000) = &H80000000 Then
      HiWord = ((lThis And &H7FFF0000) \ &H10000) Or &H8000&
   Else
      HiWord = (lThis And &HFFFF0000) \ &H10000
   End If
End Property

'set the most significant 16 bits of a long value
Public Property Let HiWord(ByRef lThis As Long, ByVal lHiWord As Long)
   If (lHiWord And &H8000&) = &H8000& Then
      lThis = lThis And Not &HFFFF0000 Or ((lHiWord And &H7FFF&) * &H10000) Or &H80000000
   Else
      lThis = lThis And Not &HFFFF0000 Or (lHiWord * &H10000)
   End If
End Property

